// Genetic object shader

#version 130

#pragma vertex_shader

in vec4 vertex;
in vec3 normal;
in vec3 tangent;
in vec2 texCoord;


#ifdef SKINNED
in vec4  weights;
in ivec4 indices;
uniform mat4 boneMatrices[70];
#endif

uniform mat4  transform;
uniform mat4  modelMatrix;

out vec2 texcoord;
out vec3 worldPos;

out vec3 oTangent;
out vec3 oBinormal;
out vec3 oNormal;

void main() {
	vec4 pos = vertex;
	vec3 norm = normal;
	vec3 tang = tangent;
	
	#ifdef SKINNED
	vec4 weight = weights;
	ivec4 index = indices;
	vec3 blendPosition = vec3(0);
	vec3 blendNormal   = vec3(0);
	vec3 blendTangent  = vec3(0);
	for(int i=0; i<SKINNED; ++i) {
		blendPosition  += (boneMatrices[index.x] * vertex).xyz * weight.x;
		blendNormal    += mat3(boneMatrices[index.x]) * normal * weight.x;
		blendTangent   += mat3(boneMatrices[index.x]) * tangent * weight.x;
		weight = weight.yzwx;
		index = index.yzwx;
	}
	pos.xyz = blendPosition;
	norm = blendNormal;
	tang = blendTangent;
	#endif

	gl_Position = transform * pos;
	texcoord = texCoord;
	worldPos = (modelMatrix * pos).xyz;

	oNormal = mat3(modelMatrix) * norm;
	oTangent = mat3(modelMatrix) * tang;
	oBinormal = cross(oNormal, oTangent);
}


#pragma fragment_shader

in vec2 texcoord;
in vec3 worldPos;

in vec3 oTangent;
in vec3 oBinormal;
in vec3 oNormal;

out vec4 fragment;

uniform sampler2D diffuse;
uniform sampler2D normal;
uniform vec3 lightDirection;

#ifdef SHADOW
uniform sampler2D shadow;
uniform mat4 shadowMatrix;
#endif


void main() {
	vec4 diff = texture(diffuse, texcoord.st);
	vec3 norm = texture(normal, texcoord.st).rgb * 2.0 - 1.0;
	norm.z = 1.0 - length(norm.xy);
	vec3 worldNormal = norm.x*oTangent - norm.y*oBinormal + norm.z*oNormal;

	#ifdef SHADOW
	vec4 shadowCoord = shadowMatrix * vec4(worldPos, 1);
	vec4 shadowValue = texture(shadow, shadowCoord.xy * 0.5 + 0.5); // should apply bias to shadowMatrix
	float shade = step(shadowCoord.z/shadowCoord.w, shadowValue.r+0.001);
	#endif


	fragment = diff;

	float l = dot( normalize(worldNormal), normalize(lightDirection));
	float s = (l+1)/1.3*0.2+0.1; // dark side lighting

	fragment.rgb *= max(s, l);

	// debug
	#ifdef SHADOW
	fragment.rgb  *= shade*0.5+0.5;
	#endif
}

